/**
 * Copyright 2011 Thorsten Höger, Taimos GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package de.taimos.jsonrpc.client;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLConnection;

import de.taimos.jsonrpc.msg.IRequest;
import de.taimos.jsonrpc.msg.IResponse;
import de.taimos.jsonrpc.msg.NotificationRequest;
import de.taimos.jsonrpc.msg.Parser;
import de.taimos.jsonrpc.msg.RPCRequest;

/**
 * This class provides a generic HTTP connector to be used for RPC clients
 * 
 * @author Thorsten Hoeger
 */
public class HTTPServerConnector implements IServerConnector {
	
	private String serverURL;
	

	/**
	 * Initiate a new HTTP ServerConnector for the given URL endpoint
	 * 
	 * @param _serverURL - the URL of the JSON-RPC servlet/endpoint
	 */
	public HTTPServerConnector(String _serverURL) {
		this.serverURL = _serverURL;
	}
	
	@Override
	public void doNotify(final NotificationRequest _notification) {
		this.execRPC(_notification);
	}
	
	@Override
	public IResponse doRPC(final RPCRequest _notification) {
		String resultString = this.execRPC(_notification);
		return Parser.readResponse(resultString);
	}
	
	/**
	 * Call this method to execute an RPC call with the given request
	 * 
	 * @param _request - the JSON-RPC request
	 * @return the result as {@link String}
	 */
	public String execRPC(final IRequest _request) {
		try {
			// Send data to server
			final URL url = new URL(this.serverURL);
			final URLConnection conn = url.openConnection();
			// prepare connection for POST
			conn.setDoInput(true);
			conn.setDoOutput(true);
			conn.setUseCaches(false);
			final OutputStreamWriter wr = new OutputStreamWriter(conn.getOutputStream());
			// Write request as JSON
			wr.write(Parser.writeObject(_request));
			wr.flush();
			
			// Get the response
			final BufferedReader rd = new BufferedReader(new InputStreamReader(conn.getInputStream()));
			final StringBuilder sb = new StringBuilder();
			String line;
			while ((line = rd.readLine()) != null) {
				sb.append(line);
			}
			wr.close();
			rd.close();
			// Return the response to the caller
			if (!sb.toString().isEmpty()) {
				return sb.toString();
			}
		} catch (final Exception e) {
			e.printStackTrace();
		}
		return "";
	}
}
