/**
 * Copyright 2011 Thorsten Höger, Taimos GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package de.taimos.jsonrpc.client;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.HashMap;

import javassist.util.proxy.MethodHandler;
import javassist.util.proxy.ProxyFactory;
import javassist.util.proxy.ProxyObject;
import de.taimos.jsonrpc.annotations.RPCClass;
import de.taimos.jsonrpc.annotations.RPCMethod;
import de.taimos.jsonrpc.annotations.RPCParam;
import de.taimos.jsonrpc.msg.ErrorResponse;
import de.taimos.jsonrpc.msg.IResponse;
import de.taimos.jsonrpc.msg.Parser;
import de.taimos.jsonrpc.msg.RPCRequest;
import de.taimos.jsonrpc.msg.SuccessResponse;

/**
 * Use this class to create a proxy for your desired RPC endpoint
 * 
 * @author Thorsten Hoeger
 */
public class RPCProxyFactory {
	
	/**
	 * Creates a proxy for the supplied interface via JSON-RPC using the
	 * supplied {@link IServerConnector}
	 * 
	 * @param <T> - The generic type of the proxy
	 * @param _iface - the class of the proxy to create
	 * @param _serverConnector - the {@link IServerConnector} to use to connect
	 *            to the server
	 * @return the created proxy instance
	 */
	public static <T> T createProxy(final Class<T> _iface, final IServerConnector _serverConnector) {
		return RPCProxyFactory.createProxy(_iface, _serverConnector, true);
	}
	
	/**
	 * Creates a proxy for the supplied interface via JSON-RPC using the
	 * supplied {@link IServerConnector}
	 * 
	 * @param <T> - The generic type of the proxy
	 * @param _iface - the class of the proxy to create
	 * @param _serverConnector - the {@link IServerConnector} to use to connect
	 *            to the server
	 * @param _useNumericID - specify whether the system should use numeric or
	 *            string values as request id
	 * @return the created proxy instance
	 */
	@SuppressWarnings("unchecked")
	public static <T> T createProxy(final Class<T> _iface, final IServerConnector _serverConnector, final boolean _useNumericID) {
		try {
			// Build proxy factory for given class
			ProxyFactory pf = new ProxyFactory();
			pf.setInterfaces(new Class[] {_iface});
			
			// Create proxy and set handler
			Object proxy = pf.createClass().newInstance();
			ProxyObject po = (ProxyObject) proxy;
			po.setHandler(new MethodHandler() {
				
				@Override
				public Object invoke(Object proxyInstance, Method call, Method superCall, Object[] params) throws Throwable {
					// Set defaults for methdo signature
					String className = _iface.getAnnotation(RPCClass.class).name();
					String methodName = call.getName();
					Class<?> returnType = call.getReturnType();
					
					// Return values for inherited methods from Object to
					// prevent RPC calls
					if (methodName.equals("finalize")) {
						return Void.TYPE;
					} else if (methodName.equals("hashCode")) {
						return this.hashCode();
					} else if (methodName.equals("equals")) {
						return false;
					} else if (methodName.equals("getClass")) {
						return this.getClass();
					} else if (methodName.equals("toString")) {
						return this.toString();
					}
					
					// Find Method annotation and search for custom return type
					RPCMethod m = call.getAnnotation(RPCMethod.class);
					if (m != null) {
						methodName = m.name();
						if (!m.contentType().equals(Object.class)) {
							returnType = m.contentType();
						}
					}
					
					// Build map with parameters
					HashMap<String, Object> parameterMap = new HashMap<String, Object>();
					if (call.getParameterTypes().length > 0) {
						Annotation[][] paramAnnotations = call.getParameterAnnotations();
						for (int i = 0; i < paramAnnotations.length; i++) {
							Annotation[] annotations = paramAnnotations[i];
							for (Annotation a : annotations) {
								if (a instanceof RPCParam) {
									parameterMap.put(((RPCParam) a).name(), params[i]);
									break;
								}
							}
						}
					}
					
					// Build request
					RPCRequest rpcRequest = new RPCRequest(className + "." + methodName, parameterMap, _useNumericID);
					
					// Execute RPC via supplied serevr connector
					IResponse response = _serverConnector.doRPC(rpcRequest);
					
					// For Success parse result data
					if (response instanceof SuccessResponse) {
						SuccessResponse success = (SuccessResponse) response;
						
						// Check ids
						// If string id is used use equals
						if (!_useNumericID && !success.getId().equals(rpcRequest.getId())) {
							throw new RuntimeException("id matching error");
						}
						// If numeric id is used compare longValue
						if (_useNumericID && (((Number) success.getId()).longValue() != ((Number) rpcRequest.getId()).longValue())) {
							throw new RuntimeException("id matching error");
						}
						
						// Return void on void methods
						if (returnType.equals(Void.TYPE)) {
							return Void.TYPE;
						}
						return Parser.convertObject(success.getResult(), returnType);
					} else if (response instanceof ErrorResponse) {
						// Throw exception for error message
						ErrorResponse error = (ErrorResponse) response;
						throw error.getAsException();
					}
					return null;
				}
				
			});
			return (T) po;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
}
