/**
 * Copyright 2011 Thorsten Höger, Taimos GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package de.taimos.jsonrpc.msg;

import java.lang.reflect.InvocationTargetException;
import java.text.MessageFormat;

import org.codehaus.jackson.annotate.JsonIgnore;
import org.codehaus.jackson.annotate.JsonProperty;

/**
 * This class represents a received error
 * 
 * @author Thorsten Hoeger
 */
public class ErrorResponse implements IResponse {
	
	static class Error {
		
		@JsonProperty
		long code;
		
		@JsonProperty
		String message;
		
		@JsonProperty
		Object data;
		
	}
	

	@JsonProperty("id")
	private Object id;
	
	@JsonProperty("error")
	private Error error;
	

	/**
	 * Creates a new empty error message
	 */
	public ErrorResponse() {
		//
	}
	
	/**
	 * Creates a new error message with the given values
	 * 
	 * @param _id - the original message id
	 * @param _code - the error code
	 * @param _message - the error message
	 * @param _data - additional error data
	 */
	public ErrorResponse(Object _id, long _code, String _message, Object _data) {
		this.id = _id;
		this.error = new Error();
		
		this.error.code = _code;
		this.error.message = _message;
		this.error.data = _data;
	}
	
	@Override
	@JsonProperty("jsonrpc")
	public String getVersion() {
		return "2.0";
	}
	
	/**
	 * Setter for the version string. This HAS to be "2.0". Otherwise an
	 * {@link IllegalArgumentException} is thrown.
	 * 
	 * @param version
	 */
	@JsonProperty("jsonrpc")
	public void checkVersion(String version) {
		if (!version.equals("2.0")) {
			throw new IllegalArgumentException();
		}
	}
	
	@Override
	@JsonIgnore
	public Object getId() {
		return this.id;
	}
	
	/**
	 * This method returns the {@link ErrorResponse} as an {@link Exception} to
	 * use try-catch-constructs
	 * 
	 * @return this error as exception
	 */
	@JsonIgnore
	public Exception getAsException() {
		String msg = MessageFormat.format("{0} ({1}), {2}", this.error.message, this.error.code, this.error.data.toString());
		return new InvocationTargetException(new Exception(msg));
	}
	
}
