/**
 * Copyright 2011 Thorsten Höger, Taimos GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package de.taimos.jsonrpc.msg;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.codehaus.jackson.map.ObjectMapper;

/**
 * This class is used to create and parse JSON
 * 
 * @author Thorsten Hoeger
 */
public class Parser {
	
	private static ObjectMapper mapper = new ObjectMapper();
	

	/**
	 * Parse response from JSON string
	 * 
	 * @param _json - the JSON string
	 * @return the response
	 */
	public static IResponse readResponse(String _json) {
		try {
			// Success messages contain result tag
			if (_json.contains("\"result\"")) {
				return Parser.mapper.readValue(_json, SuccessResponse.class);
			}
			// Error messages contain error tag
			if (_json.contains("\"error\"")) {
				return Parser.mapper.readValue(_json, ErrorResponse.class);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * Parse request from JSON string
	 * 
	 * @param _json - the JSON string
	 * @return the request
	 */
	public static IRequest readRequest(String _json) {
		try {
			// Synchronous Requests contain id tag
			if (_json.contains("\"id\"")) {
				return Parser.mapper.readValue(_json, RPCRequest.class);
			}
			return Parser.mapper.readValue(_json, NotificationRequest.class);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * Serialize the given object to JSON
	 * 
	 * @param _object - the object to serialize
	 * @return the JSON String
	 */
	public static String writeObject(Object _object) {
		try {
			return new ObjectMapper().writeValueAsString(_object);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return "";
	}
	
	/**
	 * Convert an object into another type. If the source object is a collection
	 * and the target class defines a collection component type a new collection
	 * is created and the components are converted
	 * 
	 * @param _source - the source object
	 * @param _target - the target class or collection component class
	 * @return the converted object
	 */
	public static Object convertObject(Object _source, Class<?> _target) {
		try {
			// return object if type is matching
			if (_target.isInstance(_source)) {
				return _source;
			}
			
			// If the source object is a collection and target is not then
			// interpret target as component target
			if ((_source instanceof Collection) && !_target.isArray() && !Collection.class.isAssignableFrom(_target)) {
				Collection<?> collection = (Collection<?>) _source;
				// Create new collection of same type
				@SuppressWarnings("unchecked")
				Collection<Object> newCollection = (Collection<Object>) _source.getClass().newInstance();
				
				// Add converted contents of collection to new collection
				for (Object object : collection) {
					newCollection.add(Parser.mapper.convertValue(object, _target));
				}
				return newCollection;
			}
			// Convert object
			return Parser.mapper.convertValue(_source, _target);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * Parse a batch request from JSON string
	 * 
	 * @param _json - the JSON string
	 * @return the list of requests
	 */
	@SuppressWarnings("unchecked")
	public static List<IRequest> readBatchRequest(String _json) {
		try {
			List<IRequest> requests = new ArrayList<IRequest>();
			// Parse batch request
			ArrayList<Object> batch = Parser.mapper.readValue(_json, ArrayList.class);
			for (Object request : batch) {
				// create list of requests
				requests.add(Parser.readRequest(Parser.mapper.writeValueAsString(request)));
			}
			return requests;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
}
