/**
 * Copyright 2011 Thorsten Höger, Taimos GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package de.taimos.jsonrpc.msg;

import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.atomic.AtomicLong;

import org.codehaus.jackson.annotate.JsonCreator;
import org.codehaus.jackson.annotate.JsonProperty;

/**
 * This class represents a JSON-RPC synchronous request
 * 
 * @author Thorsten Hoeger
 */
public class RPCRequest implements IRequest {
	
	private static AtomicLong idSequence = new AtomicLong(1);
	
	private String method = "";
	
	private HashMap<String, Object> params = null;
	
	private Object id;
	

	/**
	 * Create a new JSON-RPC request with given target method and idStyle
	 * 
	 * @param _method - the target method
	 * @param _useNumericID - true for numeric id, false for string
	 */
	public RPCRequest(String _method, boolean _useNumericID) {
		this(_method, null, _useNumericID);
	}
	
	/**
	 * Create a new JSON-RPC request with given target method, map of invocation
	 * parameters and idStyle
	 * 
	 * @param _method - the target method
	 * @param _params - the invocation parameters
	 * @param _useNumericID - true for numeric id, false for string
	 */
	public RPCRequest(String _method, HashMap<String, Object> _params, boolean _useNumericID) {
		this.method = _method;
		this.params = _params;
		if (_useNumericID) {
			this.id = RPCRequest.idSequence.getAndIncrement();
		} else {
			this.id = UUID.randomUUID().toString();
		}
	}
	
	/**
	 * Create a new JSON-RPC request with given target method, map of invocation
	 * parameters and fixed numeric id
	 * 
	 * @param _method - the target method
	 * @param _params - the invocation parameters
	 * @param _id - the id to use
	 */
	public RPCRequest(String _method, HashMap<String, Object> _params, long _id) {
		this.method = _method;
		this.params = _params;
		this.id = _id;
	}
	
	/**
	 * Create a new JSON-RPC request with given target method, map of invocation
	 * parameters and fixed string id
	 * 
	 * @param _method - the target method
	 * @param _params - the invocation parameters
	 * @param _id - the id to use
	 */
	public RPCRequest(String _method, HashMap<String, Object> _params, String _id) {
		this.method = _method;
		this.params = _params;
		this.id = _id;
	}
	
	/**
	 * Create a new JSON-RPC request with given target method, map of invocation
	 * parameters and fixed id
	 * 
	 * @param _method - the target method
	 * @param _params - the invocation parameters
	 * @param _id - the id to use
	 */
	@JsonCreator
	public RPCRequest(@JsonProperty("method") String _method, @JsonProperty("params") HashMap<String, Object> _params, @JsonProperty("id") Object _id) {
		this.method = _method;
		this.params = _params;
		this.id = _id;
	}
	
	@Override
	@JsonProperty("jsonrpc")
	public String getVersion() {
		// This is given in the spec
		return "2.0";
	}
	
	/**
	 * Setter for the version string. This HAS to be "2.0". Otherwise an
	 * {@link IllegalArgumentException} is thrown.
	 * 
	 * @param version
	 */
	@JsonProperty("jsonrpc")
	public void checkVersion(String version) {
		if (!version.equals("2.0")) {
			throw new IllegalArgumentException();
		}
	}
	
	@Override
	public String getMethod() {
		return this.method;
	}
	
	@Override
	public HashMap<String, Object> getParams() {
		return this.params;
	}
	
	/**
	 * Returns the request id for future use in the associated response
	 * 
	 * @return the call id as {@link String} or {@link Long}
	 */
	public Object getId() {
		return this.id;
	}
	
}
