/**
 * Copyright 2011 Thorsten Höger, Taimos GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package de.taimos.jsonrpc.server;

import java.lang.annotation.Annotation;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Hashtable;

import de.taimos.jsonrpc.annotations.RPCClass;
import de.taimos.jsonrpc.annotations.RPCMethod;
import de.taimos.jsonrpc.annotations.RPCParam;
import de.taimos.jsonrpc.msg.ErrorResponse;
import de.taimos.jsonrpc.msg.IRequest;
import de.taimos.jsonrpc.msg.IResponse;
import de.taimos.jsonrpc.msg.Parser;
import de.taimos.jsonrpc.msg.RPCRequest;
import de.taimos.jsonrpc.msg.SuccessResponse;

/**
 * This class is used as an RPC endpoint. Instantiate this class with an object
 * to create a JSON-RPC wrapper for this given object
 * 
 * @author Thorsten Hoeger
 */
public class ClassRequestHandler {
	
	private Hashtable<String, Method> methodTargets = new Hashtable<String, Method>();
	
	private Hashtable<String, Class<?>> methodReturnTypes = new Hashtable<String, Class<?>>();
	
	private Hashtable<String, String[]> paramNameMap = new Hashtable<String, String[]>();
	
	private Hashtable<String, Class<?>[]> paramTargetTypeMap = new Hashtable<String, Class<?>[]>();
	
	private Object invocationTarget;
	

	/**
	 * Create a new {@link ClassRequestHandler} for the given invocation target
	 * 
	 * @param _target - the invocation target
	 */
	public ClassRequestHandler(Object _target) {
		this.invocationTarget = _target;
		
		// Scan the class for annotated interface
		Class<?> rpcClass = this.findRPCClass(_target.getClass());
		
		Method[] methods = rpcClass.getMethods();
		// Scan methods
		for (Method method : methods) {
			String name = method.getName();
			Class<?> returnType = method.getReturnType();
			if (method.getAnnotation(RPCMethod.class) != null) {
				// scan method name
				name = method.getAnnotation(RPCMethod.class).name();
				// scan for custom return type
				if (!method.getAnnotation(RPCMethod.class).contentType().equals(Object.class)) {
					returnType = method.getAnnotation(RPCMethod.class).contentType();
				}
			}
			this.methodTargets.put(name, method);
			this.methodReturnTypes.put(name, returnType);
			
			Annotation[][] paramAnnotations = method.getParameterAnnotations();
			Class<?>[] paramTypes = method.getParameterTypes();
			
			// Scan parameters
			String[] paramNames = new String[paramTypes.length];
			for (int i = 0; i < paramAnnotations.length; i++) {
				Annotation[] annotations = paramAnnotations[i];
				String paramName = null;
				for (Annotation a : annotations) {
					if (a instanceof RPCParam) {
						// Set parameter name
						paramName = ((RPCParam) a).name();
						// Set custom parameter type
						if (!((RPCParam) a).contentType().equals(Object.class)) {
							paramTypes[i] = ((RPCParam) a).contentType();
						}
						break;
					}
				}
				paramNames[i] = paramName;
			}
			this.paramNameMap.put(name, paramNames);
			this.paramTargetTypeMap.put(name, paramTypes);
		}
	}
	
	private Class<?> findRPCClass(Class<?> clazz) {
		if (clazz.isAnnotationPresent(RPCClass.class)) {
			return clazz;
		}
		// Scan interfaces
		for (Class<?> iface : clazz.getInterfaces()) {
			Class<?> cl = this.findRPCClass(iface);
			if (cl != null) {
				return cl;
			}
		}
		return null;
	}
	
	/**
	 * Invokes the given request on the object represented by this wrapper
	 * 
	 * @param _request - the request to execute
	 * @return the execution result
	 * @throws Throwable if anything goes wrong
	 */
	public IResponse invoke(IRequest _request) throws Throwable {
		String methodName = _request.getMethod().split("\\.")[1];
		Method method = this.methodTargets.get(methodName);
		// If method does not exist return error
		if (method == null) {
			if (_request instanceof RPCRequest) {
				return new ErrorResponse(((RPCRequest) _request).getId(), -32601, "Method not found", _request.getMethod());
			}
			System.out.println("Notification with illegal method name: " + _request.getMethod());
			return null;
		}
		
		String[] paramNames = this.paramNameMap.get(methodName);
		Class<?>[] paramTypes = this.paramTargetTypeMap.get(methodName);
		Object[] params = new Object[paramNames.length];
		
		// Prepare parameters
		if (paramNames.length > 0) {
			HashMap<String, Object> jsonParams = _request.getParams();
			// If number of supplied params and number of expected params does
			// not match throw error
			if (paramNames.length != jsonParams.size()) {
				if (_request instanceof RPCRequest) {
					return new ErrorResponse(((RPCRequest) _request).getId(), -32602, "Invalid params", "");
				}
				System.out.println("Notification with illegal params: " + _request.getParams());
				return null;
			}
			// convert parameter to matching type
			for (int i = 0; i < paramNames.length; i++) {
				params[i] = jsonParams.get(paramNames[i]);
				params[i] = Parser.convertObject(params[i], paramTypes[i]);
			}
		}
		
		try {
			// Invoke the method
			Object res = method.invoke(this.invocationTarget, params);
			
			// If call was synchronous return the invocation result
			if (_request instanceof RPCRequest) {
				RPCRequest rpc = (RPCRequest) _request;
				return new SuccessResponse(rpc.getId(), res);
			}
			
			return null;
		} catch (InvocationTargetException e) {
			throw e.getTargetException();
		}
	}
}
