/**
 * Copyright 2011 Thorsten Höger, Taimos GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */
package de.taimos.jsonrpc.server;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.InetSocketAddress;
import java.util.concurrent.Executors;

import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpServer;

/**
 * USe this class a simple HTTP endpoint for JSON-RPC calls
 * 
 * @author Thorsten Hoeger
 */
@SuppressWarnings("restriction")
public class HTTPServer {

	private final String host;

	private final int port;

	RequestHandler requestHandler = new RequestHandler();

	private HttpServer httpServer;

	/**
	 * Create new HTTP Server on 0.0.0.0:8080
	 */
	public HTTPServer() {
		this("0.0.0.0", 8080);
	}

	/**
	 * Create new HTTP Server on the given interface and port
	 * 
	 * @param _host
	 *            - the host to register the server on
	 * @param _port
	 *            - the port to use
	 */
	public HTTPServer(String _host, int _port) {
		this.host = _host;
		this.port = _port;
	}

	/**
	 * Add an invocation target to this endpoint
	 * 
	 * @param _handler
	 *            - the RPC handler
	 */
	public void addHandler(Object _handler) {
		this.requestHandler.register(_handler);
	}

	/**
	 * Starts the HTTP server at URL: http://host:port/json-rpc
	 */
	public void startServer() {
		try {
			if (this.httpServer != null) {
				// Already started
				return;
			}
			// create new server instance
			this.httpServer = HttpServer.create(new InetSocketAddress(this.host, this.port), 5);
			this.httpServer.setExecutor(Executors.newFixedThreadPool(5));
			this.httpServer.start();

			// create handler for HTTP server
			this.httpServer.createContext("/json-rpc", new HttpHandler() {

				@Override
				public void handle(HttpExchange ex) throws IOException {
					// Get the request
					final BufferedReader rd = new BufferedReader(new InputStreamReader(ex.getRequestBody()));
					final StringBuilder sb = new StringBuilder();
					String line;
					while ((line = rd.readLine()) != null) {
						sb.append(line);
					}
					rd.close();

					// execute the request via the request handler
					final String response = HTTPServer.this.requestHandler.invoke(sb.toString());

					// return the response to the client
					ex.sendResponseHeaders(HttpURLConnection.HTTP_OK, response.length());
					ex.getResponseBody().write(response.getBytes());
					ex.close();
				}
			});
		} catch (final IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Stops the server
	 */
	public void stopServer() {
		if (this.httpServer != null) {
			this.httpServer.stop(0);
			this.httpServer = null;
		}
	}

}
