/**
 * Copyright 2011 Thorsten Höger, Taimos GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package de.taimos.jsonrpc.server;

import java.util.ArrayList;
import java.util.Hashtable;

import de.taimos.jsonrpc.annotations.RPCClass;
import de.taimos.jsonrpc.msg.ErrorResponse;
import de.taimos.jsonrpc.msg.IRequest;
import de.taimos.jsonrpc.msg.IResponse;
import de.taimos.jsonrpc.msg.Parser;
import de.taimos.jsonrpc.msg.RPCRequest;

/**
 * This class is used as a handler registry. Instantiate this class and register
 * invocation targets with it. Then use it to invoke methods on the supplied
 * objects
 * 
 * @author Thorsten Hoeger
 */
public class RequestHandler {
	
	private Hashtable<String, ClassRequestHandler> invocationTargets = new Hashtable<String, ClassRequestHandler>();
	

	/**
	 * Create a new JSON-RPC endpoint registry
	 */
	public RequestHandler() {
		//
	}
	
	/**
	 * Register a new invocation target in the registry
	 * 
	 * @param _target - the invocation target to register
	 * @return the class name, the target was registered on
	 */
	public String register(Object _target) {
		// Scna for the name of this class
		String className = this.findClassName(_target.getClass());
		// Remove already existing class with given name
		if (this.invocationTargets.containsKey(className)) {
			this.invocationTargets.remove(className);
		}
		// Add handler to target map
		this.invocationTargets.put(className, new ClassRequestHandler(_target));
		return className;
	}
	
	private String findClassName(Class<?> clazz) {
		if (clazz.isAnnotationPresent(RPCClass.class)) {
			return clazz.getAnnotation(RPCClass.class).name();
		}
		// Scan interfaces
		for (Class<?> iface : clazz.getInterfaces()) {
			String name = this.findClassName(iface);
			if (name != null) {
				return name;
			}
		}
		return null;
	}
	
	/**
	 * Invokes the given request on the according object
	 * 
	 * @param _request - the JSON-RPC request
	 * @return the response
	 */
	public String invoke(String _request) {
		// check if request is batch
		if (_request.startsWith("[")) {
			ArrayList<IResponse> arr = new ArrayList<IResponse>();
			// Execute every request and create response list
			for (IRequest req : Parser.readBatchRequest(_request)) {
				if (req != null) {
					IResponse res = this.invoke(req);
					if (res != null) {
						arr.add(res);
					}
				}
			}
			// write list to client
			return Parser.writeObject(arr);
		}
		if (_request.startsWith("{")) {
			// execute request
			IResponse r = this.invoke(Parser.readRequest(_request));
			try {
				// return response
				String response = Parser.writeObject(r);
				return response;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return "";
		
	}
	
	/**
	 * Invokes the given request on the according object
	 * 
	 * @param _request - the JSON-RPC request
	 * @return the response
	 */
	public IResponse invoke(IRequest _request) {
		IResponse response = null;
		
		// Split rpc method into class and method name
		String className = _request.getMethod().split("\\.")[0];
		Object id = 0;
		if (_request instanceof RPCRequest) {
			// Set id for synchronous requests
			id = ((RPCRequest) _request).getId();
		}
		
		// return error if class is not registered
		if (!this.invocationTargets.containsKey(className)) {
			response = new ErrorResponse(id, -32601, "Method not found", _request.getMethod());
		} else {
			// find handler for given class
			ClassRequestHandler crh = this.invocationTargets.get(className);
			try {
				// invoke given method
				response = crh.invoke(_request);
			} catch (Throwable e) {
				response = new ErrorResponse(id, -32603, "Internal error", e.getMessage());
			}
		}
		
		// return response
		if (_request instanceof RPCRequest) {
			return response;
		}
		return null;
	}
}
